
CREATE OR REPLACE FUNCTION media (
		numerador IN NUMBER,
		denominador IN NUMBER )
	RETURN NUMBER DETERMINISTIC
AS
BEGIN
	IF ( denominador <> 0 ) THEN
		RETURN ( numerador / denominador ) ;
	ELSE
		RETURN ( 0 ) ;
	END IF;
END;
/

CREATE TABLE CategoriaGasto
	(
		idCategoriaGasto INTEGER NOT NULL,
		descripcion      VARCHAR2 ( 50 ) NOT NULL,
		idTipoGasto      INTEGER NOT NULL
	) ;
ALTER TABLE CategoriaGasto ADD CONSTRAINT CategoriaGasto_PK PRIMARY KEY ( idCategoriaGasto ) ;

CREATE TABLE Ciudad
	(
		idCiudad INTEGER NOT NULL,
		nombre   VARCHAR2 ( 50 ) NOT NULL,
		idRegion INTEGER NOT NULL
	) ;
ALTER TABLE Ciudad ADD CONSTRAINT Ciudad_PK PRIMARY KEY ( idCiudad ) ;

CREATE TABLE Ejercicio
	(
		idEjercicio INTEGER NOT NULL,
		inicio      DATE NOT NULL,
		fin         DATE NOT NULL,
		cerradoSN   CHAR ( 1 ) NOT NULL
	) ;
ALTER TABLE Ejercicio ADD CONSTRAINT Ejercicio_PK PRIMARY KEY ( idEjercicio ) ;

CREATE TABLE Estado
	(
		idEstado    INTEGER NOT NULL,
		descripcion VARCHAR2 ( 50 ) NOT NULL,
		FormaPagoSN CHAR ( 1 ) NOT NULL
	) ;
ALTER TABLE Estado ADD CONSTRAINT Estado_PK PRIMARY KEY ( idEstado ) ;

CREATE TABLE FormaPago
	(
		idFormaPago INTEGER NOT NULL,
		descripcion VARCHAR2 ( 50 ) NOT NULL
	) ;
ALTER TABLE FormaPago ADD CONSTRAINT FormaPago_PK PRIMARY KEY ( idFormaPago ) ;

CREATE TABLE Gasto
	(
		idGasto          INTEGER NOT NULL,
		fecha            DATE NOT NULL,
		idEjercicio      INTEGER NOT NULL,
		idParlamento     INTEGER NOT NULL,
		idParlamentario  INTEGER,
		idPartido        INTEGER,
		descripcion      VARCHAR2 ( 500 ),
		baseImponible    NUMBER DEFAULT 0.0 NOT NULL,
		idTipoIVA        INTEGER NOT NULL,
		idCategoriaGasto INTEGER NOT NULL,
		idEstado         INTEGER NOT NULL,
		idFormaPago      INTEGER,
		idProveedor      INTEGER,
		justificacion    VARCHAR2 ( 500 ),
		presupuestoURL   VARCHAR2 ( 200 ),
		facturaURL       VARCHAR2 ( 200 ),
		contratoURL      VARCHAR2 ( 200 )
	) ;
ALTER TABLE Gasto ADD CONSTRAINT Gasto_PK PRIMARY KEY ( idGasto ) ;

CREATE TABLE Pais
	(
		idPais INTEGER NOT NULL,
		nombre VARCHAR2 ( 50 ) NOT NULL
	) ;
ALTER TABLE Pais ADD CONSTRAINT Pais_PK PRIMARY KEY ( idPais ) ;

CREATE TABLE Parlamentario
	(
		idParlamentario INTEGER NOT NULL,
		nombre          VARCHAR2 ( 50 ) NOT NULL,
		apellidos       VARCHAR2 ( 50 ) NOT NULL
	) ;
ALTER TABLE Parlamentario ADD CONSTRAINT Parlamentario_PK PRIMARY KEY ( idParlamentario ) ;

CREATE TABLE ParlamentarioST
	(
		idEjercicio     INTEGER NOT NULL,
		idParlamento    INTEGER NOT NULL,
		idParlamentario INTEGER NOT NULL,
		totalGastos     NUMBER DEFAULT 0.0 NOT NULL
	) ;
ALTER TABLE ParlamentarioST ADD CONSTRAINT ParlamentarioST_PK PRIMARY KEY ( idEjercicio, idParlamento, idParlamentario ) ;

CREATE TABLE Parlamento
	(
		idParlamento      INTEGER NOT NULL,
		nombre            VARCHAR2 ( 50 ) NOT NULL,
		idCiudad          INTEGER NOT NULL,
		idTipoParlamento  INTEGER NOT NULL,
		direccion         VARCHAR2 ( 50 ) NOT NULL,
		codigoPostal      VARCHAR2 ( 20 ) NOT NULL,
		telefono          VARCHAR2 ( 20 ),
		web               VARCHAR2 ( 200 ),
		maxParlamentarios INTEGER NOT NULL
	) ;
ALTER TABLE Parlamento ADD CONSTRAINT Parlamento_PK PRIMARY KEY ( idParlamento ) ;

CREATE TABLE ParlamentoDH
	(
		idParlamentoDH  INTEGER NOT NULL,
		idParlamento    INTEGER NOT NULL,
		idParlamentario INTEGER NOT NULL,
		desde           DATE NOT NULL,
		hasta           DATE
	) ;
ALTER TABLE ParlamentoDH ADD CONSTRAINT ParlamentoDH_PK PRIMARY KEY ( idParlamentoDH ) ;

CREATE TABLE ParlamentoST
	(
		idEjercicio               INTEGER NOT NULL,
		idParlamento              INTEGER NOT NULL,
		totalGastosParlamentarios NUMBER DEFAULT 0.0 NOT NULL,
		totalGastosGenerales      NUMBER DEFAULT 0.0 NOT NULL,
		totalGastos               NUMBER AS ( totalGastosParlamentarios + totalGastosGenerales ) VIRTUAL NOT NULL,
		numParlamentarios         INTEGER DEFAULT 0 NOT NULL,
		minGastoParlamentario     NUMBER DEFAULT 0.0 NOT NULL,
		mediaGastoParlamentario   NUMBER AS ( media ( totalGastosParlamentarios, numParlamentarios ) ) VIRTUAL NOT NULL,
		maxGastoParlamentario     NUMBER DEFAULT 0.0 NOT NULL
	) ;
ALTER TABLE ParlamentoST ADD CONSTRAINT ParlamentoST_PK PRIMARY KEY ( idEjercicio, idParlamento ) ;

CREATE TABLE Partido
	(
		idPartido INTEGER NOT NULL,
		nombre    VARCHAR2 ( 50 ) NOT NULL
	) ;
ALTER TABLE Partido ADD CONSTRAINT Partido_PK PRIMARY KEY ( idPartido ) ;

CREATE TABLE PartidoDH
	(
		idPartidoDH     INTEGER NOT NULL,
		idParlamentario INTEGER NOT NULL,
		idPartido       INTEGER NOT NULL,
		desde           DATE NOT NULL,
		hasta           DATE
	) ;
ALTER TABLE PartidoDH ADD CONSTRAINT PartidoDH_PK PRIMARY KEY ( idPartidoDH ) ;

CREATE TABLE Proveedor
	(
		idProveedor INTEGER NOT NULL,
		nombre      VARCHAR2 ( 50 ) NOT NULL,
		CIF         VARCHAR2 ( 20 ) NOT NULL
	) ;
ALTER TABLE Proveedor ADD CONSTRAINT Proveedor_PK PRIMARY KEY ( idProveedor ) ;
ALTER TABLE Proveedor ADD CONSTRAINT Proveedor_CIF_UN UNIQUE ( CIF ) ;

CREATE TABLE Region
	(
		idRegion INTEGER NOT NULL,
		nombre   VARCHAR2 ( 50 ) NOT NULL,
		idPais   INTEGER NOT NULL
	) ;
ALTER TABLE Region ADD CONSTRAINT Region_PK PRIMARY KEY ( idRegion ) ;

CREATE TABLE Registro
	(
		idRegistro    INTEGER NOT NULL,
		procedimiento VARCHAR2 ( 100 ) NOT NULL,
		usuario       VARCHAR2 ( 50 ) NOT NULL,
		instante      TIMESTAMP NOT NULL,
		entrada       VARCHAR2 ( 3000 ),
		salida        VARCHAR2 ( 3000 ) NOT NULL
	) ;
ALTER TABLE Registro ADD CONSTRAINT Registro_PK PRIMARY KEY ( idRegistro ) ;

CREATE TABLE TipoGasto
	(
		idTipoGasto     INTEGER NOT NULL,
		descripcion     VARCHAR2 ( 50 ) NOT NULL,
		parlamentarioSN CHAR ( 1 ) NOT NULL
	) ;
ALTER TABLE TipoGasto ADD CONSTRAINT TipoGasto_PK PRIMARY KEY ( idTipoGasto ) ;

CREATE TABLE TipoIVA
	(
		idTipoIVA     INTEGER NOT NULL,
		PorcentajeIVA NUMBER DEFAULT 0.0 NOT NULL
	) ;
ALTER TABLE TipoIVA ADD CONSTRAINT TipoIVA_PK PRIMARY KEY ( idTipoIVA ) ;

CREATE TABLE TipoParlamento
	(
		idTipoParlamento INTEGER NOT NULL,
		descripcion      VARCHAR2 ( 50 ) NOT NULL
	) ;
ALTER TABLE TipoParlamento ADD CONSTRAINT TipoParlamento_PK PRIMARY KEY ( idTipoParlamento ) ;

ALTER TABLE CategoriaGasto ADD CONSTRAINT CategoriaGasto_TipoGasto_FK FOREIGN KEY ( idTipoGasto ) REFERENCES TipoGasto ( idTipoGasto ) ;

ALTER TABLE Ciudad ADD CONSTRAINT Ciudad_Region_FK FOREIGN KEY ( idRegion ) REFERENCES Region ( idRegion ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_CategoriaGasto_FK FOREIGN KEY ( idCategoriaGasto ) REFERENCES CategoriaGasto ( idCategoriaGasto ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Ejercicio_FK FOREIGN KEY ( idEjercicio ) REFERENCES Ejercicio ( idEjercicio ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Estado_FK FOREIGN KEY ( idEstado ) REFERENCES Estado ( idEstado ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_FormaPago_FK FOREIGN KEY ( idFormaPago ) REFERENCES FormaPago ( idFormaPago ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Parlamentario_FK FOREIGN KEY ( idParlamentario ) REFERENCES Parlamentario ( idParlamentario ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Parlamento_FK FOREIGN KEY ( idParlamento ) REFERENCES Parlamento ( idParlamento ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Partido_FK FOREIGN KEY ( idPartido ) REFERENCES Partido ( idPartido ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_Proveedor_FK FOREIGN KEY ( idProveedor ) REFERENCES Proveedor ( idProveedor ) ;

ALTER TABLE Gasto ADD CONSTRAINT Gasto_TipoIVA_FK FOREIGN KEY ( idTipoIVA ) REFERENCES TipoIVA ( idTipoIVA ) ;

ALTER TABLE ParlamentarioST ADD CONSTRAINT ParlamentarioST_Ejercicio_FK FOREIGN KEY ( idEjercicio ) REFERENCES Ejercicio ( idEjercicio ) ;

ALTER TABLE ParlamentarioST ADD CONSTRAINT ParlamentarioST_Parlamento_FK FOREIGN KEY ( idParlamento ) REFERENCES Parlamento ( idParlamento ) ;

ALTER TABLE ParlamentarioST ADD CONSTRAINT ParlamentarioST_id_FK FOREIGN KEY ( idParlamentario ) REFERENCES Parlamentario ( idParlamentario ) ;

ALTER TABLE ParlamentoDH ADD CONSTRAINT ParlamentoDH_Parlamentario_FK FOREIGN KEY ( idParlamentario ) REFERENCES Parlamentario ( idParlamentario ) ;

ALTER TABLE ParlamentoDH ADD CONSTRAINT ParlamentoDH_Parlamento_FK FOREIGN KEY ( idParlamento ) REFERENCES Parlamento ( idParlamento ) ;

ALTER TABLE ParlamentoST ADD CONSTRAINT ParlamentoST_Ejercicio_FK FOREIGN KEY ( idEjercicio ) REFERENCES Ejercicio ( idEjercicio ) ;

ALTER TABLE ParlamentoST ADD CONSTRAINT ParlamentoST_Parlamento_FK FOREIGN KEY ( idParlamento ) REFERENCES Parlamento ( idParlamento ) ;

ALTER TABLE Parlamento ADD CONSTRAINT Parlamento_Ciudad_FK FOREIGN KEY ( idCiudad ) REFERENCES Ciudad ( idCiudad ) ;

ALTER TABLE Parlamento ADD CONSTRAINT Parlamento_TipoParlamento_FK FOREIGN KEY ( idTipoParlamento ) REFERENCES TipoParlamento ( idTipoParlamento ) ;

ALTER TABLE PartidoDH ADD CONSTRAINT PartidoDH_Parlamentario_FK FOREIGN KEY ( idParlamentario ) REFERENCES Parlamentario ( idParlamentario ) ;

ALTER TABLE PartidoDH ADD CONSTRAINT PartidoDH_Partido_FK FOREIGN KEY ( idPartido ) REFERENCES Partido ( idPartido ) ;

ALTER TABLE Region ADD CONSTRAINT Region_Pais_FK FOREIGN KEY ( idPais ) REFERENCES Pais ( idPais ) ;

CREATE SEQUENCE idCategoriaGasto_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idCategoriaGasto BEFORE
	INSERT ON CategoriaGasto FOR EACH ROW WHEN
		(
			NEW.idCategoriaGasto IS NULL
		)
		BEGIN :NEW.idCategoriaGasto := idCategoriaGasto_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idCiudad_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idCiudad BEFORE
	INSERT ON Ciudad FOR EACH ROW WHEN
		(
			NEW.idCiudad IS NULL
		)
		BEGIN :NEW.idCiudad := idCiudad_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idEstado_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idEstado BEFORE
	INSERT ON Estado FOR EACH ROW WHEN
		(
			NEW.idEstado IS NULL
		)
		BEGIN :NEW.idEstado := idEstado_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idFormaPago_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idFormaPago BEFORE
	INSERT ON FormaPago FOR EACH ROW WHEN
		(
			NEW.idFormaPago IS NULL
		)
		BEGIN :NEW.idFormaPago := idFormaPago_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idGasto_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idGasto BEFORE
	INSERT ON Gasto FOR EACH ROW WHEN
		(
			NEW.idGasto IS NULL
		)
		BEGIN :NEW.idGasto := idGasto_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idPais_SQ START WITH 1 MINVALUE 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idPais BEFORE
	INSERT ON Pais FOR EACH ROW WHEN
		(
			NEW.idPais IS NULL
		)
		BEGIN :NEW.idPais := idPais_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idParlamentario_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idParlamentario BEFORE
	INSERT ON Parlamentario FOR EACH ROW WHEN
		(
			NEW.idParlamentario IS NULL
		)
		BEGIN :NEW.idParlamentario := idParlamentario_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idParlamento_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idParlamento BEFORE
	INSERT ON Parlamento FOR EACH ROW WHEN
		(
			NEW.idParlamento IS NULL
		)
		BEGIN :NEW.idParlamento := idParlamento_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idParlamentoDH_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idParlamentoDH BEFORE
	INSERT ON ParlamentoDH FOR EACH ROW WHEN
		(
			NEW.idParlamentoDH IS NULL
		)
		BEGIN :NEW.idParlamentoDH := idParlamentoDH_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idPartido_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idPartido BEFORE
	INSERT ON Partido FOR EACH ROW WHEN
		(
			NEW.idPartido IS NULL
		)
		BEGIN :NEW.idPartido := idPartido_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idPartidoDH_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idPartidoDH BEFORE
	INSERT ON PartidoDH FOR EACH ROW WHEN
		(
			NEW.idPartidoDH IS NULL
		)
		BEGIN :NEW.idPartidoDH := idPartidoDH_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idProveedor_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idProveedor BEFORE
	INSERT ON Proveedor FOR EACH ROW WHEN
		(
			NEW.idProveedor IS NULL
		)
		BEGIN :NEW.idProveedor := idProveedor_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idRegion_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idRegion BEFORE
	INSERT ON Region FOR EACH ROW WHEN
		(
			NEW.idRegion IS NULL
		)
		BEGIN :NEW.idRegion := idRegion_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idRegistro_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER id BEFORE
	INSERT ON Registro FOR EACH ROW WHEN
		(
			NEW.idRegistro IS NULL
		)
		BEGIN :NEW.idRegistro := idRegistro_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idTipoGasto_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idTipoGasto BEFORE
	INSERT ON TipoGasto FOR EACH ROW WHEN
		(
			NEW.idTipoGasto IS NULL
		)
		BEGIN :NEW.idTipoGasto := idTipoGasto_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idTipoIVA_SQ START WITH 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idTipoIVA BEFORE
	INSERT ON TipoIVA FOR EACH ROW WHEN
		(
			NEW.idTipoIVA IS NULL
		)
		BEGIN :NEW.idTipoIVA := idTipoIVA_SQ.NEXTVAL;
END;
/

CREATE SEQUENCE idTipoParlamento_SQ START WITH 1 MINVALUE 1 NOCACHE ORDER ;
CREATE OR REPLACE TRIGGER idTipoParlamento BEFORE
	INSERT ON TipoParlamento FOR EACH ROW WHEN
		(
			NEW.idTipoParlamento IS NULL
		)
		BEGIN :NEW.idTipoParlamento := idTipoParlamento_SQ.NEXTVAL;
END;
/
