CREATE OR REPLACE PACKAGE     pkg_estadistica AS

/**
Dado un parlamento, un ltimo ejercicio y el nmero de ejercicios anteriores a tener en cuenta calcula la suma total de gastos 
de ese parlamento durante ese perodo.

@param p_idUltimoEjercicio                Identificador del ltimo ejercicio.
@param p_numEjerciciosAnteriores          Nmero de ejercicios anteriores al ltimo a tener en cuenta.
@param p_idParlamento                     Identificador del parlamento.
@param p_sumaTotalGastos                  Devuelve la suma total de gastos.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE totalGastos_parlamento (
  p_idUltimoEjercicio       IN  ParlamentoST.idEjercicio % TYPE,
  p_numEjerciciosAnteriores IN  NATURAL,
  p_idParlamento            IN  ParlamentoST.idParlamento % TYPE,
  p_sumaTotalGastos         OUT ParlamentoST.totalGastos % TYPE,
  p_rsp                     OUT VARCHAR2);

/**
Dado un ejercicio calcula qu parlamento ha tenido un total de gastos ms bajo.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamento                     Devuelve el identificador del parlamento con el total de gastos ms bajo.
@param p_minTotalGastos                   Devuelve la suma total de gastos ms baja.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE minTotalGastos_parlamentos (
  p_idEjercicio    IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento   OUT ParlamentoST.idParlamento % TYPE,
  p_minTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp            OUT VARCHAR2);

/**
Dado un ejercicio calcula qu parlamento ha tenido un total de gastos ms alto.

@param p_idEjercicio                      Identificador del ejercicio a tener en cuenta.
@param p_idParlamento                     Devuelve el identificador del parlamento con el total de gastos ms alto.
@param p_minTotalGastos                   Devuelve la suma total de gastos ms alta.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE maxTotalGastos_parlamentos (
  p_idEjercicio    IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento   OUT ParlamentoST.idParlamento % TYPE,
  p_maxTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp            OUT VARCHAR2);

/**
Dado un ejercicio calcula la suma total de los gastos de todos los parlamentos durante ese ejercicio.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_maxTotalGastos                   Devuelve la suma total de gastos de todos los parlamentos ese ejercicio.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE sumaTotalGastos_parlamentos (
  p_idEjercicio     IN  ParlamentoST.idEjercicio % TYPE,
  p_sumaTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Dado un parlamentario, un ltimo ejercicio y el nmero de ejercicios anteriores a tener en cuenta calcula la suma total de gastos 
atribuibles a ese parlamentario durante ese perodo.

@param p_idUltimoEjercicio                Identificador del ltimo ejercicio.
@param p_numEjerciciosAnteriores          Nmero de ejercicios anteriores al ltimo a tener en cuenta.
@param p_idParlamentario                  Identificador del parlamentario.
@param p_sumaTotalGastos                  Devuelve la suma total de gastos del parlamentario.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE totalGastos_parlamentario (
  p_idUltimoEjercicio       IN  ParlamentarioST.idEjercicio % TYPE,
  p_numEjerciciosAnteriores IN  NATURAL,
  p_idParlamentario         IN  ParlamentarioST.idParlamentario % TYPE,
  p_sumaTotalGastos         OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp                     OUT VARCHAR2);

/**
Dado un ejercicio calcula qu parlamentario ha tenido un total de gastos ms bajo.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamentario                  Devuelve el identificador del parlamentario con el total de gastos ms bajo.
@param p_minTotalGastos                   Devuelve la suma total de gastos ms baja.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE minTotalGastos_parlamentario (
  p_idEjercicio     IN  ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario OUT ParlamentarioST.idParlamentario % TYPE,
  p_minTotalGastos  OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Dado un ejercicio calcula qu parlamentario ha tenido un total de gastos ms alto.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamentario                  Devuelve el identificador del parlamentario con el total de gastos ms alto.
@param p_maxTotalGastos                   Devuelve la suma total de gastos ms alta.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE maxTotalGastos_parlamentario (
  p_idEjercicio     IN  ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario OUT ParlamentarioST.idParlamentario % TYPE,
  p_maxTotalGastos  OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2);

/**
Dado un ejercicio y un parlamento, calcula la diferencia entre el parlamentario que tiene ms gastos imputados y el
parlamentario que tiene menos gastos imputados.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamento                     Identificacor del parlamento.
@param p_minGastoParlamentario            Devuelve el importe del gasto mnimo de un parlamentario en ese ejercicio y parlamento.
@param p_maxGastoParlamentario            Devuelve el importe del gasto mximo de un parlamentario en ese ejercicio y parlamento.
@parma p_diferenciaGastoParlamentario     Devuelve la diferencia en entre el gasto mximo y mnimo de un parlamentario en ese ejercicio y parlamento.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE diferenciaGastoParlamentario (
  p_idEjercicio                  IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento                 IN  ParlamentoST.idParlamento % TYPE,
  p_minGastoParlamentario        OUT ParlamentoST.minGastoParlamentario % TYPE,
  p_maxGastoParlamentario        OUT ParlamentoST.maxGastoParlamentario % TYPE,
  p_diferenciaGastoParlamentario OUT ParlamentoST.totalGastos % TYPE,
  p_rsp                          OUT VARCHAR2);

/**
Dado un ejercicio, calcula la media del gasto de un parlamentario durante ese ejercicio teniendo en cuenta a todos los
parlamentarios de todos los parlamentos.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_sumaGastosParlamentarios         Devuelve la suma de todos los gastos imputables a parlamentarios durante el ejercicio.
@parma p_sumaParlamentarios               Devuelve el nmero de todos los parlamentarios durante el ejercicio.
@param p_mediaGastosParlamentarios        Devuelve la media del gasto imputable a parlamentarios durante el ejercicio.
*/
PROCEDURE mediaGastosParlamentarios (
  p_idEjercicio               IN  ParlamentoST.idEjercicio % TYPE,
  p_sumaGastosParlamentarios  OUT ParlamentoST.totalGastosParlamentarios % TYPE,
  p_sumaParlamentarios        OUT ParlamentoST.numParlamentarios % TYPE,
  p_mediaGastosParlamentarios OUT ParlamentoST.totalGastosParlamentarios % TYPE,
  p_rsp                       OUT VARCHAR2);

/**
Dado un ejercicio, calcula el parlamento con la media de gasto por parlamentario ms baja durante ese ejercicio.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamento                     Devuelve el identificador del parlamento con una media de gasto por parlamentario ms baja.
@param p_nombre                           Devuelve el nombre del parlamento con una media de gasto por parlamentario ms baja.
@param p_minMediaGastoParlamentario       Devuelve la media de gasto por parlamentario ms baja.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE minMediaGastoParlamentario (
  p_idEjercicio                IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento               OUT ParlamentoST.idParlamento % TYPE,
  p_nombre                     OUT Parlamento.nombre % TYPE,
  p_minMediaGastoParlamentario OUT ParlamentoST.mediaGastoParlamentario % TYPE,
  p_rsp                        OUT VARCHAR2);

/**
Dado un ejercicio, calcula el parlamento con la media de gasto por parlamentario ms alta durante ese ejercicio.

@param p_idEjercicio                      Identificador del ejercicio.
@param p_idParlamento                     Devuelve el identificador del parlamento con una media de gasto por parlamentario ms alta.
@param p_nombre                           Devuelve el nombre del parlamento con una media de gasto por parlamentario ms alta.
@param p_maxMediaGastoParlamentario       Devuelve la media de gasto por parlamentario ms alta.
@param p_rsp                              Devuelve el resultado de la ejecucin del procedimiento.
*/
PROCEDURE maxMediaGastoParlamentario (
  p_idEjercicio                IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento               OUT ParlamentoST.idParlamento % TYPE,
  p_nombre                     OUT Parlamento.nombre % TYPE,
  p_maxMediaGastoParlamentario OUT ParlamentoST.mediaGastoParlamentario % TYPE,
  p_rsp                        OUT VARCHAR2);

END pkg_estadistica;
/


CREATE OR REPLACE PACKAGE BODY     pkg_estadistica AS

entrada Registro.entrada % TYPE;	-- String con los parmetros de entrada y sus valores.
salida Registro.salida % TYPE;		-- String con los parmetros de salida y sus valores.

--
-- Dado un parlamento, un ltimo ejercicio y el nmero de ejercicios anteriores a tener en cuenta calcula la suma total de gastos 
-- de ese parlamento durante ese perodo.
--
PROCEDURE totalGastos_parlamento (
  p_idUltimoEjercicio       IN  ParlamentoST.idEjercicio % TYPE,
  p_numEjerciciosAnteriores IN  NATURAL,
  p_idParlamento            IN  ParlamentoST.idParlamento % TYPE,
  p_sumaTotalGastos         OUT ParlamentoST.totalGastos % TYPE,
  p_rsp                     OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      -- Controla que p_idUltimoEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idUltimoEjercicio);
      -- Controla que existan estadisticas para este Ejercicio y este Parlamento.
      pkg_comun.existe_ParlamentoST (p_idUltimoEjercicio, p_idParlamento);
      -- Calcula la suma de totalGastos de este Parlamento en los ltimos ejercicios solicitados existentes.
      SELECT
        SUM (totalGastos)
      INTO
        p_sumaTotalGastos
      FROM
        ParlamentoST
      WHERE
        idParlamento = p_idParlamento
        AND
        (p_idUltimoEjercicio - p_numEjerciciosAnteriores) <= idEjercicio
        AND
        idEjercicio <= p_idUltimoEjercicio;
    EXCEPTION
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ltimo ejercicio indicado [' || p_idUltimoEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idUltimoEjercicio || '] y parlamento [' || p_idParlamento || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idUltimoEjercicio = [' || p_idUltimoEjercicio || '] + numEjerciciosAnteriores = [' || p_numEjerciciosAnteriores || '] + idParlamento = [' || p_idParlamento || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'sumaTotalGastos = [' || p_sumaTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.totalGastos_parlamento', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END totalGastos_parlamento;

--
-- Dado un ejercicio calcula qu parlamento ha tenido un total de gastos ms bajo.
--
PROCEDURE minTotalGastos_parlamentos (
  p_idEjercicio    IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento   OUT ParlamentoST.idParlamento % TYPE,
  p_minTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp            OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        MIN (totalGastos)
      INTO
        p_minTotalGastos
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamento pertenece ese minTotalGastos.
      -- Podra existir ms de un Parlamento cuyo totalGastos este Ejercicio coincida con minTotalGastos.
      -- En ese caso elegiremos el de menor idParlamento.
      SELECT
        MIN (idParlamento)
      INTO
        p_idParlamento
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        totalGastos = p_minTotalGastos;
    EXCEPTION
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamento = [' || p_idParlamento || '] + minTotalGastos = [' || p_minTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.minTotalGastos_parlamentos', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END minTotalGastos_parlamentos;

--
-- Dado un ejercicio calcula qu parlamento ha tenido un total de gastos ms alto.
--
PROCEDURE maxTotalGastos_parlamentos (
  p_idEjercicio    IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento   OUT ParlamentoST.idParlamento % TYPE,
  p_maxTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp            OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        MAX (totalGastos)
      INTO
        p_maxTotalGastos
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamento pertenece ese maxTotalGastos.
      -- Podra existir ms de un Parlamento cuyo totalGastos este Ejercicio coincida con maxTotalGastos.
      -- En ese caso elegiremos el de menor idParlamento.
      SELECT
        MIN (idParlamento)
      INTO
        p_idParlamento
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        totalGastos = p_maxTotalGastos;
    EXCEPTION
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamento = [' || p_idParlamento || '] + maxTotalGastos = [' || p_maxTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.maxTotalGastos_parlamentos', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END maxTotalGastos_parlamentos;

--
-- Dado un ejercicio calcula la suma total de los gastos de todos los parlamentos durante ese ejercicio.
--
PROCEDURE sumaTotalGastos_parlamentos (
  p_idEjercicio     IN  ParlamentoST.idEjercicio % TYPE,
  p_sumaTotalGastos OUT ParlamentoST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        SUM (totalGastos)
      INTO
        p_sumaTotalGastos
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'sumaTotalGastos = [' || p_sumaTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.sumaTotalGastos_parlamentos', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END sumaTotalGastos_parlamentos;

--
-- Dado un parlamentario, un ltimo ejercicio y el nmero de ejercicios anteriores a tener en cuenta calcula la suma total de gastos 
-- atribuibles a ese parlamentario durante ese perodo.
--
PROCEDURE totalGastos_parlamentario (
  p_idUltimoEjercicio       IN  ParlamentarioST.idEjercicio % TYPE,
  p_numEjerciciosAnteriores IN  NATURAL,
  p_idParlamentario         IN  ParlamentarioST.idParlamentario % TYPE,
  p_sumaTotalGastos         OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp                     OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idParlamentario exista en la base de datos.
      pkg_comun.existe_parlamentario (p_idParlamentario);
      -- Controla que p_idUltimoEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idUltimoEjercicio);
      -- Controla que existan estadisticas para este Ejercicio y este Parlamentario.
      pkg_comun.existe_ParlamentarioST (p_idUltimoEjercicio, p_idParlamentario);
      -- Calcula la suma de totalGastos de este Parlamentario en los ltimos ejercicios solicitados existentes.
      SELECT
        SUM (totalGastos)
      INTO
        p_sumaTotalGastos
      FROM
        ParlamentarioST
      WHERE
        idParlamentario = p_idParlamentario
        AND
        (p_idUltimoEjercicio - p_numEjerciciosAnteriores) <= idEjercicio
        AND
        idEjercicio <= p_idUltimoEjercicio;
    EXCEPTION
      WHEN pkg_comun.ErrorParlamentarioNoExiste THEN
        p_rsp := 'ERROR: El parlamentario [' || p_idParlamentario || '] no existe.';
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ltimo ejercicio indicado [' || p_idUltimoEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idUltimoEjercicio || '] y parlamentario [' || p_idParlamentario || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idUltimoEjercicio = [' || p_idUltimoEjercicio || '] + numEjerciciosAnteriores = [' || p_numEjerciciosAnteriores || '] + idParlamentario = [' || p_idParlamentario || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'sumaTotalGastos = [' || p_sumaTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.totalGastos_parlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END totalGastos_parlamentario;

--
-- Dado un ejercicio calcula qu parlamentario ha tenido un total de gastos ms bajo.
--
PROCEDURE minTotalGastos_parlamentario (
  p_idEjercicio     IN  ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario OUT ParlamentarioST.idParlamentario % TYPE,
  p_minTotalGastos  OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentarioST (p_idEjercicio);
      -- Calcula el mnimo totalGastos de este Ejercicio.
      SELECT
        MIN (totalGastos)
      INTO
        p_minTotalGastos
      FROM
        ParlamentarioST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamentario pertenece ese minTotalGastos.
      -- Podra existir ms de un Parlamentario cuyo totalGastos este Ejercicio coincida con minTotalGastos.
      -- En ese caso elegiremos el de menor idParlamentario.
      SELECT
        MIN (idParlamentario)
      INTO
        p_idParlamentario
      FROM
        ParlamentarioST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        totalGastos = p_minTotalGastos;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamentario = [' || p_idParlamentario || '] + minTotalGastos = [' || p_minTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.minTotalGastos_parlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END minTotalGastos_parlamentario;

--
-- Dado un ejercicio calcula qu parlamentario ha tenido un total de gastos ms alto.
--
PROCEDURE maxTotalGastos_parlamentario (
  p_idEjercicio     IN  ParlamentarioST.idEjercicio % TYPE,
  p_idParlamentario OUT ParlamentarioST.idParlamentario % TYPE,
  p_maxTotalGastos  OUT ParlamentarioST.totalGastos % TYPE,
  p_rsp             OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentarioST (p_idEjercicio);
      -- Calcula el mximo totalGastos de este Ejercicio.
      SELECT
        MAX (totalGastos)
      INTO
        p_maxTotalGastos
      FROM
        ParlamentarioST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamentario pertenece ese maxTotalGastos.
      -- Podra existir ms de un Parlamentario cuyo totalGastos este Ejercicio coincida con maxTotalGastos.
      -- En ese caso elegiremos el de menor idParlamentario.
      SELECT
        MIN (idParlamentario)
      INTO
        p_idParlamentario
      FROM
        ParlamentarioST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        totalGastos = p_maxTotalGastos;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamentario = [' || p_idParlamentario || '] + maxTotalGastos = [' || p_maxTotalGastos || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.maxTotalGastos_parlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END maxTotalGastos_parlamentario;

--
-- Dado un ejercicio y un parlamento, calcula la diferencia entre el parlamentario que tiene ms gastos imputados y el
-- parlamentario que tiene menos gastos imputados.
--
PROCEDURE diferenciaGastoParlamentario (
  p_idEjercicio                  IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento                 IN  ParlamentoST.idParlamento % TYPE,
  p_minGastoParlamentario        OUT ParlamentoST.minGastoParlamentario % TYPE,
  p_maxGastoParlamentario        OUT ParlamentoST.maxGastoParlamentario % TYPE,
  p_diferenciaGastoParlamentario OUT ParlamentoST.totalGastos % TYPE,
  p_rsp                          OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idParlamento exista en la base de datos.
      pkg_comun.existe_parlamento (p_idParlamento);
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio y este Parlamento.
      pkg_comun.existe_ParlamentoST (p_idEjercicio, p_idParlamento);
      -- Consulta la estadstica este Ejercicio y Parlamento.
      SELECT
        minGastoParlamentario,
        maxGastoParlamentario
      INTO
        p_minGastoParlamentario,
        p_maxGastoParlamentario
      FROM
        ParlamentoST
      WHERE
        idParlamento = p_idParlamento
        AND
        idEjercicio = p_idejercicio;
      p_diferenciaGastoParlamentario := p_maxGastoParlamentario - p_minGastoParlamentario;
    EXCEPTION
      WHEN pkg_comun.ErrorParlamentoNoExiste THEN
        p_rsp := 'ERROR: El parlamento [' || p_idParlamento || '] no existe.';
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || '] y parlamento [' || p_idParlamento || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || '] + idParlamento = [' || p_idParlamento || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'minGastoParlamentario = [' || p_minGastoParlamentario || '] + maxGastoParlamentario = [' || p_maxGastoParlamentario || '] + maxTotalGastos_parlamentario = [' || p_diferenciaGastoParlamentario || ' + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.maxTotalGastos_parlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END diferenciaGastoParlamentario;

--
-- Dado un ejercicio, calcula la media del gasto de un parlamentario durante ese ejercicio teniendo en cuenta a todos los
-- parlamentarios de todos los parlamentos.
--
PROCEDURE mediaGastosParlamentarios (
  p_idEjercicio               IN  ParlamentoST.idEjercicio % TYPE,
  p_sumaGastosParlamentarios  OUT ParlamentoST.totalGastosParlamentarios % TYPE,
  p_sumaParlamentarios        OUT ParlamentoST.numParlamentarios % TYPE,
  p_mediaGastosParlamentarios OUT ParlamentoST.totalGastosParlamentarios % TYPE,
  p_rsp                       OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        SUM (totalGastosParlamentarios),
        SUM (numParlamentarios)
      INTO
        p_sumaGastosParlamentarios,
        p_sumaParlamentarios
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
      p_mediaGastosParlamentarios := MEDIA (p_sumaGastosParlamentarios, p_sumaParlamentarios);
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'sumaGastosParlamentarios = [' || p_sumaGastosParlamentarios || '] + sumaParlamentarios = [' || p_sumaParlamentarios || '] + mediaGastosParlamentarios = [' || p_mediaGastosParlamentarios || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.mediaGastosParlamentarios', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END mediaGastosParlamentarios;

--
-- Dado un ejercicio, calcula el parlamento con la media de gasto por parlamentario ms baja durante ese ejercicio.
--
PROCEDURE minMediaGastoParlamentario (
  p_idEjercicio                IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento               OUT ParlamentoST.idParlamento % TYPE,
  p_nombre                     OUT Parlamento.nombre % TYPE,
  p_minMediaGastoParlamentario OUT ParlamentoST.mediaGastoParlamentario % TYPE,
  p_rsp                        OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        MIN (mediaGastoParlamentario)
      INTO
        p_minMediaGastoParlamentario
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamento pertenece ese minMediaGastoParlamentario.
      -- Podra existir ms de un Parlamento cuya mediaGastoParlamentario este Ejercicio coincida con minMediaGastoParlamentario.
      -- En ese caso elegiremos el de menor idParlamento.
      SELECT
        MIN (idParlamento)
      INTO
        p_idParlamento
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        mediaGastoParlamentario = p_minMediaGastoParlamentario;
      -- Consulta el parlamento.
      SELECT
        nombre
      INTO
        p_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamento = [' || p_idParlamento || '] + nombre = [' || p_nombre || '] + minMediaGastoParlamentario = [' || p_minMediaGastoParlamentario || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.minMediaGastoParlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END minMediaGastoParlamentario;

--
-- Dado un ejercicio, calcula el parlamento con la media de gasto por parlamentario ms alta durante ese ejercicio.
--
PROCEDURE maxMediaGastoParlamentario (
  p_idEjercicio                IN  ParlamentoST.idEjercicio % TYPE,
  p_idParlamento               OUT ParlamentoST.idParlamento % TYPE,
  p_nombre                     OUT Parlamento.nombre % TYPE,
  p_maxMediaGastoParlamentario OUT ParlamentoST.mediaGastoParlamentario % TYPE,
  p_rsp                        OUT VARCHAR2)
AS
BEGIN
  BEGIN
    -- Inicializa el resultado de ejecucin del procedimiento a OK.
    p_rsp := 'OK';
    BEGIN
      -- Controla que p_idEjercicio exista en la base de datos.
      pkg_comun.existe_ejercicio (p_idEjercicio);
      -- Controla que existan estadisticas para este Ejercicio.
      pkg_comun.existe_ParlamentoST (p_idEjercicio);
      -- Consulta la estadstica este Ejercicio.
      SELECT
        MAX (mediaGastoParlamentario)
      INTO
        p_maxMediaGastoParlamentario
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio;
      -- Busca a qu Parlamento pertenece esta maxMediaGastoParlamentario.
      -- Podra existir ms de un Parlamento cuya mediaGastoParlamentario este Ejercicio coincida con maxMediaGastoParlamentario.
      -- En ese caso elegiremos el de menor idParlamento.
      SELECT
        MIN (idParlamento)
      INTO
        p_idParlamento
      FROM
        ParlamentoST
      WHERE
        idEjercicio = p_idEjercicio
        AND
        mediaGastoParlamentario = p_maxMediaGastoParlamentario;
      -- Consulta el parlamento.
      SELECT
        nombre
      INTO
        p_nombre
      FROM
        Parlamento
      WHERE
        idParlamento = p_idParlamento;
    EXCEPTION
      WHEN pkg_comun.ErrorEjercicioNoExiste THEN
        p_rsp := 'ERROR: El ejercicio [' || p_idEjercicio || '] no existe.';
      WHEN pkg_comun.ErrorEstadisticaNoExiste THEN
        p_rsp := 'ERROR: No existe estadstica para el ejercicio [' || p_idEjercicio || ']';
      WHEN OTHERS THEN
        p_rsp := 'ERROR: ' || SQLERRM;
    END;
    -- Construye el string con los parmetros de entrada y sus valores.
    entrada := 'idEjercicio = [' || p_idEjercicio || ']';
    -- Construye el string con los parmetros de salida y sus valores.
    salida := 'idParlamento = [' || p_idParlamento || '] + nombre = [' || p_nombre || '] + maxMediaGastoParlamentario = [' || p_maxMediaGastoParlamentario || '] + rsp = ' || p_rsp;
    -- Registra la ejecucin del procedimiento junto con sus parmetros de entrada y salida.
    pkg_comun.entrada_registro ('pkg_estadistica.maxMediaGastoParlamentario', USER, SYSTIMESTAMP, entrada, salida);
  EXCEPTION
    WHEN OTHERS THEN
      DBMS_OUTPUT.PUT_LINE ('ERROR INSERTANDO ENTRADA EN REGISTRO: ' || SQLERRM);
  END;
END maxMediaGastoParlamentario;

END pkg_estadistica;
/
